import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A rocket that can be controlled by the arrowkeys: up, left, right.
 * The gun is fired by hitting the 'space' key. 'z' releases a proton wave.
 * 
 * @author Poul Henriksen
 * @author Michael Kolling
 * 
 * @version 1.0
 */
public class Rocket extends SmoothMover
{
    private static final int gunReloadTime = 5;         // The minimum delay between firing the gun.
    private static final int protonReloadTime = 900;

    private int reloadDelayCount;               // How long ago we fired the gun the last time.
    private int protonDelayCount;
    
    private GreenfootImage rocket = new GreenfootImage("rocket.png");    
    private GreenfootImage rocketWithThrust = new GreenfootImage("rocketWithThrust.png");

    /**
     * Initilise this rocket.
     */
    public Rocket()
    {
        reloadDelayCount = 5;
        protonDelayCount = 900;
        Vector v = new Vector(Greenfoot.getRandomNumber(361), 0.4);
        this.addForce(v);
    }

    /**
     * Do what a rocket's gotta do. (Which is: mostly flying about, and turning,
     * accelerating and shooting when the right keys are pressed.)
     */
    public void act()
    {
        move();
        checkKeys();
        reloadDelayCount++;
        protonDelayCount++;
        checkProtonDelayCount();
        checkForCollision();
    }
    
    /**
     * Check whether there are any key pressed and react to them.
     */
    private void checkKeys() 
    {
        if (Greenfoot.isKeyDown("space")) 
        {
            fire();
        }
        if (Greenfoot.isKeyDown("right")) 
        {
            turn(5);
        }
        if (Greenfoot.isKeyDown("left")) 
        {
            turn(-5);
        }
        if (Greenfoot.isKeyDown("down"))
        {
            startProtonWave();
        }
        
        ignite(Greenfoot.isKeyDown("up"));
    }
    
    /**
     * Check to see if we hit an astroid
     */
    public void checkForCollision()
    {
        Actor a = getOneIntersectingObject(Asteroid.class);
        if (a != null)
        {
            Space s = (Space) getWorld();
            Explosion e = new Explosion();
            s.addObject(e, this.getX() , this.getY());
            s.gameOver();
            s.removeObject(this);
        }
    }
    
    private void ignite(boolean boosterOn)
    {
        if (boosterOn)
        {
            this.setImage("rocketWithThrust.png");
            Vector v = new Vector(this.getRotation(), 0.2); 
            this.addForce(v);
        }
        else
        {
            this.setImage("rocket.png");
        }
    }
    
    /**
     * Fire a bullet if the gun is ready.
     */
    private void fire() 
    {
        if (reloadDelayCount >= gunReloadTime) 
        {
            Bullet bullet = new Bullet (getMovement().copy(), getRotation());
            getWorld().addObject (bullet, getX(), getY());
            bullet.move ();
            reloadDelayCount = 0;
        }
    }
    
    private void startProtonWave()
    {
        if (protonDelayCount >= protonReloadTime)
        {
            Space s = (Space) getWorld();
            ProtonWave p = new ProtonWave();
            s.addObject(p, this.getX() , this.getY());
            protonDelayCount = 0;
        }
    }
    
    private void checkProtonDelayCount()
    {
        Space s = (Space) getWorld();
        if (protonDelayCount == 900)
            s.protonIndicator();
    }
    
}