import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
import java.util.List;

/**
 * Space. Something for rockets to fly in.
 * 
 * @author Michael Kolling
 * @version 1.0
 */
public class Space extends World
{
    private Counter scoreCounter;
    private int startAsteroids = 2;

    public Space() 
    {
        super(600, 400, 1); //calling the superclass constructor
        GreenfootImage background = getBackground();
        background.setColor(Color.BLACK);
        background.fill();
        createStars(Greenfoot.getRandomNumber(12) + 17);
        
        Rocket rocket = new Rocket();
        int r = Greenfoot.getRandomNumber(150);
        int r2 = Greenfoot.getRandomNumber(150);
        addObject(rocket, getWidth()/2 + r, getHeight()/2 + r2);
        
        addAsteroids(startAsteroids);
        
        scoreCounter = new Counter("Score: ");
        addObject(scoreCounter, 60, 380);

        Explosion.initializeImages();
        ProtonWave.initializeImages();
    }
    
    public void act()
    {
        checkForAsteroids();
    }
    
    /**
     * Add a given number of asteroids to our world. Asteroids are only added into
     * the left half of the world.
     */
    private void addAsteroids(int count) 
    {
        for(int i = 0; i < count; i++) 
        {
            int x = Greenfoot.getRandomNumber(getWidth()/2);
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new Asteroid(), x, y);
        }
    }
    
    public void countScore(int score)
    {
        scoreCounter.add(score);
    }
    
    /**
     * draws stars on the background image
     *@param numStars the number of stars
     */
    private void createStars(int numStars)
    {
        for ( int i = 0; i < numStars; i++)
        {
            GreenfootImage background = getBackground();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            background.setColor(Color.WHITE);
            int r = Greenfoot.getRandomNumber(2) + 2;
            background.fillOval(x, y, r, r);
        }
    }
    
    public void checkForAsteroids()
    {
        List<Asteroid> allAsteroids = getObjects(Asteroid.class);
        if (allAsteroids.isEmpty())
        {
            Greenfoot.playSound("hooray.wav");
            scoreCounter.newLevel();
            startAsteroids++;
            addAsteroids(startAsteroids);
        }
    }
    
    public void protonIndicator()
    {
        ProtonWaveIndicator p = new ProtonWaveIndicator();
        Greenfoot.delay(5);
        removeObject(p);
    }
    
    /**
     * This method is called when the game is over to display the final score.
     */
    public void gameOver() 
    {
        ScoreBoard s = new ScoreBoard(scoreCounter.getValue(), scoreCounter.getLevel());
        this.addObject(s, this.getWidth() / 2, this.getHeight() / 2);
    }
}